#!/usr/bin/env python
# -*- coding: utf-8 -*-

import codecs
import doctest
import os
import unittest
import warnings

from vcard import vcard, vcard_definitions, vcard_errors, vcard_utils, vcard_validator, vcard_validators

TEST_DIRECTORY = os.path.dirname(__file__)


def _get_vcard_file(path):
    """
    Get the vCard contents locally or remotely.

    @param path: File relative to current directory or a URL
    @return: Text in the given file
    """
    if path in ("", None):
        return ""

    filename = os.path.join(TEST_DIRECTORY, path)

    with codecs.open(filename, "r", "utf-8") as file_pointer:
        contents = file_pointer.read()

    return contents


# vCards with errors
VCARDS_CONTINUATION_AT_START = {
    "message": vcard_errors.NOTE_CONTINUATION_AT_START,
    "vcards": ("continuation_at_start.vcf",),
}
VCARDS_DOT_AT_LINE_START = {"message": vcard_errors.NOTE_DOT_AT_LINE_START, "vcards": ("dot_at_line_start.vcf",)}
VCARDS_EMPTY_VCARD = {"message": vcard_errors.NOTE_EMPTY_VCARD, "vcards": ("", None)}
VCARDS_INVALID_DATE = {"message": vcard_errors.NOTE_INVALID_DATE, "vcards": tuple()}
VCARDS_INVALID_LANGUAGE_VALUE = {
    "message": vcard_errors.NOTE_INVALID_LANGUAGE_VALUE,
    "vcards": ("invalid_language_value.vcf",),
}
VCARDS_INVALID_LINE_SEPARATOR = {
    "message": vcard_errors.NOTE_INVALID_LINE_SEPARATOR,
    "vcards": ("line_ending_mac.vcf", "line_ending_unix.vcf", "line_ending_mixed.vcf"),
}
VCARDS_INVALID_PARAM_NAME = {"message": vcard_errors.NOTE_INVALID_PARAMETER_NAME, "vcards": ("invalid_param_name.vcf",)}
VCARDS_INVALID_PARAM_VALUE = {
    "message": vcard_errors.NOTE_INVALID_PARAMETER_VALUE,
    "vcards": ("invalid_param_value.vcf",),
}
VCARDS_INVALID_PROPERTY_NAME = {
    "message": vcard_errors.NOTE_INVALID_PROPERTY_NAME,
    "vcards": ("invalid_property_foo.vcf",),
}
VCARDS_INVALID_SUB_VALUE = {"message": vcard_errors.NOTE_INVALID_SUB_VALUE, "vcards": tuple()}
VCARDS_INVALID_SUB_VALUE_COUNT = {"message": vcard_errors.NOTE_INVALID_SUB_VALUE_COUNT, "vcards": tuple()}
VCARDS_INVALID_TEXT_VALUE = {"message": vcard_errors.NOTE_INVALID_TEXT_VALUE, "vcards": tuple()}
VCARDS_INVALID_TIME = {"message": vcard_errors.NOTE_INVALID_TIME, "vcards": tuple()}
VCARDS_INVALID_TIME_ZONE = {"message": vcard_errors.NOTE_INVALID_TIME_ZONE, "vcards": tuple()}
VCARDS_INVALID_URI = {"message": vcard_errors.NOTE_INVALID_URI, "vcards": tuple()}
VCARDS_INVALID_VALUE = {"message": vcard_errors.NOTE_INVALID_VALUE, "vcards": ("invalid_begin.vcf",)}
VCARDS_INVALID_VALUE_COUNT = {
    "message": vcard_errors.NOTE_INVALID_VALUE_COUNT,
    "vcards": ("invalid_value_count_wp.vcf",),
}
VCARDS_INVALID_X_NAME = {"message": vcard_errors.NOTE_INVALID_X_NAME, "vcards": tuple()}
VCARDS_MISMATCH_GROUP = {"message": vcard_errors.NOTE_MISMATCH_GROUP, "vcards": ("mismatch_group.vcf",)}
VCARDS_MISMATCH_PARAMETER = {"message": vcard_errors.NOTE_MISMATCH_PARAMETER, "vcards": tuple()}
VCARDS_MISSING_GROUP = {"message": vcard_errors.NOTE_MISSING_GROUP, "vcards": ("missing_group.vcf",)}
VCARDS_MISSING_PARAMETER = {"message": vcard_errors.NOTE_MISSING_PARAMETER, "vcards": ("missing_photo_param.vcf",)}
VCARDS_MISSING_PARAM_VALUE = {"message": vcard_errors.NOTE_MISSING_PARAM_VALUE, "vcards": ("missing_param_value.vcf",)}
VCARDS_MISSING_PROPERTY = {
    "message": vcard_errors.NOTE_MISSING_PROPERTY,
    "vcards": (
        "missing_properties.vcf",
        "missing_start.vcf",
        "missing_end.vcf",
        "missing_version.vcf",
        "missing_n.vcf",
        "missing_fn.vcf",
    ),
}
VCARDS_MISSING_VALUE_STRING = {"message": vcard_errors.NOTE_MISSING_VALUE_STRING, "vcards": ("missing_n_value.vcf",)}

VCARDS_WITH_ERROR = (
    VCARDS_CONTINUATION_AT_START,
    VCARDS_DOT_AT_LINE_START,
    VCARDS_EMPTY_VCARD,
    VCARDS_INVALID_DATE,
    VCARDS_INVALID_LANGUAGE_VALUE,
    VCARDS_INVALID_LINE_SEPARATOR,
    VCARDS_INVALID_PARAM_NAME,
    VCARDS_INVALID_PARAM_VALUE,
    VCARDS_INVALID_PROPERTY_NAME,
    VCARDS_INVALID_SUB_VALUE,
    VCARDS_INVALID_SUB_VALUE_COUNT,
    VCARDS_INVALID_TIME,
    VCARDS_INVALID_TIME_ZONE,
    VCARDS_INVALID_URI,
    VCARDS_INVALID_VALUE,
    VCARDS_INVALID_VALUE_COUNT,
    VCARDS_INVALID_X_NAME,
    VCARDS_MISMATCH_GROUP,
    VCARDS_MISMATCH_PARAMETER,
    VCARDS_MISSING_GROUP,
    VCARDS_MISSING_PARAMETER,
    VCARDS_MISSING_PARAM_VALUE,
    VCARDS_MISSING_PROPERTY,
    VCARDS_MISSING_VALUE_STRING,
)

# Reference cards with errors
VCARDS_REFERENCE_ERRORS = (
    # https://tools.ietf.org/html/rfc2426
    "rfc_2426_a.vcf",
    "rfc_2426_b.vcf",
)

# Valid vCards
VCARDS_VALID = ("minimal.vcf", "maximal.vcf", "scrambled_case.vcf")


class TestVCards(unittest.TestCase):
    """Test example vCards"""

    def test_failing(self):
        """vCards with errors"""
        for test_group in VCARDS_WITH_ERROR:
            for vcard_file in test_group["vcards"]:
                vcard_text = _get_vcard_file(vcard_file)
                if vcard_text is None:
                    continue

                try:
                    with warnings.catch_warnings(record=True):
                        vcard_validator.VCard(vcard_text, filename=vcard_file)
                        self.fail("Invalid vCard created:\n{0}".format(vcard_text))
                except vcard_errors.VCardError as error:
                    error_message = "\n\n".join(
                        (
                            "Wrong message for vCard {vcard_file!r}:".format(vcard_file=vcard_file),
                            "Expected: {expected}".format(expected=test_group["message"]),
                            "Got: {got}".format(got=str(error)),
                        )
                    )
                    self.assertTrue(test_group["message"] in str(error), msg=error_message)

    def test_valid(self):
        """Valid (but not necessarily sane) vCards"""
        for vcard_file in VCARDS_VALID:
            vcard_text = _get_vcard_file(vcard_file)
            if vcard_text is None:
                continue

            try:
                with warnings.catch_warnings(record=True):
                    vc_obj = vcard_validator.VCard(vcard_text, filename=vcard_file)
                self.assertNotEqual(vc_obj, None)
            except vcard_errors.VCardError as error:
                error_message = "\n\n".join(
                    (
                        "Expected valid vCard for {vcard_file!r}, but it failed to validate".format(
                            vcard_file=vcard_file
                        ),
                        str(error),
                    )
                )
                self.fail(error_message)

    def test_online(self):
        """vCards in references which are invalid"""
        for vcard_file in VCARDS_REFERENCE_ERRORS:
            vcard_text = _get_vcard_file(vcard_file)
            if vcard_text is None:
                continue

            with warnings.catch_warnings(record=True):
                self.assertRaises(vcard_errors.VCardError, vcard_validator.VCard, vcard_text)

    def test_doc(self):
        """Run DocTests"""
        self.assertEqual(doctest.testmod(vcard)[0], 0)
        self.assertEqual(doctest.testmod(vcard_definitions)[0], 0)
        self.assertEqual(doctest.testmod(vcard_errors)[0], 0)
        self.assertEqual(doctest.testmod(vcard_utils)[0], 0)
        self.assertEqual(doctest.testmod(vcard_validators)[0], 0)



from unittest import TestCase

from vcard.vcard_errors import VCardValueError
from vcard.vcard_validators import validate_date


class TestVcardValidators(TestCase):
    def test_validate_date_succeeds_with_valid_date_without_separators(self):
        date_string = "19990101"
        validate_date(date_string)

    def test_validate_date_succeeds_with_valid_date_with_separators(self):
        date_string = "1999-01-01"
        validate_date(date_string)

    def test_validate_date_succeeds_with_leap_day(self):
        date_string = "2008-02-29"
        validate_date(date_string)

    def test_validate_date_succeeds_with_leap_day_in_year_divisible_by_four_hundred(self):
        date_string = "2000-02-29"
        validate_date(date_string)

    def test_validate_date_fails_with_leap_day_in_year_not_divisible_by_four(self):
        date_string = "2010-02-29"
        self.assertRaises(VCardValueError, validate_date, date_string)

    def test_validate_date_fails_with_leap_day_in_year_divisible_by_one_hundred(self):
        date_string = "1900-02-29"
        self.assertRaises(VCardValueError, validate_date, date_string)

    def test_validate_date_fails_with_wrong_separator(self):
        date_string = "1999:01:01"
        self.assertRaises(VCardValueError, validate_date, date_string)

    def test_validate_date_fails_with_shortened_date(self):
        date_string = "1999101"
        self.assertRaises(VCardValueError, validate_date, date_string)

    def test_validate_date_fails_with_nonsense_date(self):
        date_string = "aaaa-bb-cc"
        self.assertRaises(VCardValueError, validate_date, date_string)

    def test_validate_date_error_explains_itself(self):
        date_string = "1999:01:01"
        try:
            validate_date(date_string)
        except VCardValueError as error:
            self.assertIn("Invalid date", str(error))

    def test_validate_date_error_contains_date(self):
        date_string = "1999:01:01"
        try:
            validate_date(date_string)
        except VCardValueError as error:
            self.assertIn(date_string, str(error))



import argparse
from unittest import TestCase

import mock

from vcard import vcard

ARGUMENTS_WITH_PATH = argparse.Namespace(paths=["any"], verbose=False)
ARGUMENTS_WITH_PATHS = argparse.Namespace(paths=["any", "another"], verbose=False)


class TestVcard(TestCase):
    @mock.patch("vcard.vcard.parse_arguments")
    @mock.patch("vcard.vcard.VcardValidator", spec=vcard.VcardValidator)
    def test_main_succeeds_when_vcard_validator_returns_nothing(self, vcard_validator_mock, parse_arguments_mock):
        parse_arguments_mock.return_value = ARGUMENTS_WITH_PATH
        vcard_validator_mock.return_value.result = None
        self.assertEqual(0, vcard.main())

    @mock.patch("vcard.vcard.parse_arguments")
    @mock.patch("vcard.vcard.VcardValidator", spec=vcard.VcardValidator)
    def test_main_fails_when_vcard_validator_fails(self, vcard_validator_mock, parse_arguments_mock):
        parse_arguments_mock.return_value = ARGUMENTS_WITH_PATH
        vcard_validator_mock.return_value.result = "non-empty"
        self.assertEqual(1, vcard.main())

    @mock.patch("vcard.vcard.parse_arguments")
    @mock.patch("vcard.vcard.VcardValidator")
    def test_main_fails_when_vcard_validator_fails_on_first_file(self, vcard_validator_mock, parse_arguments_mock):
        parse_arguments_mock.return_value = ARGUMENTS_WITH_PATHS
        vcard_validator_mock.side_effect = [
            mock.Mock(spec=vcard.VcardValidator, result="non-empty"),
            mock.Mock(spec=vcard.VcardValidator, result=None),
        ]
        self.assertEqual(1, vcard.main())

    @mock.patch("vcard.vcard.parse_arguments")
    def test_main_fails_when_argument_parsing_fails(self, parse_arguments_mock):
        parse_arguments_mock.side_effect = vcard.UsageError("error")
        self.assertEqual(2, vcard.main())

    def test_parse_arguments_succeeds_with_single_path(self):
        path = "/some/path"
        expected_paths = [path]

        actual_paths = vcard.parse_arguments([path]).paths

        self.assertEqual(expected_paths, actual_paths)

    def test_parse_arguments_succeeds_with_multiple_paths(self):
        path1 = "/some/path"
        path2 = "/other/path"
        arguments = [path1, path2]
        expected_paths = arguments

        actual_paths = vcard.parse_arguments(arguments).paths

        self.assertEqual(expected_paths, actual_paths)

    def test_parse_arguments_fails_without_path(self):
        self.assertRaises(SystemExit, vcard.parse_arguments, [])

    def test_parse_arguments_fails_with_invalid_argument(self):
        self.assertRaises(SystemExit, vcard.parse_arguments, ["--foo"])

    def test_parse_arguments_verbose_off_by_default(self):
        path = "/some/path"

        actual_verbosity = vcard.parse_arguments([path]).verbose

        self.assertFalse(actual_verbosity)

    def test_parse_arguments_sets_verbose_when_passed(self):
        path = "/some/path"

        actual_verbosity = vcard.parse_arguments(["--verbose", path]).verbose

        self.assertTrue(actual_verbosity)



# -*- coding: utf-8 -*-
from unittest import TestCase

from vcard.vcard_errors import VCardError


class TestVCardError(TestCase):
    def test_empty_context_output(self):
        actual = str(VCardError("message", {}))
        expected = "message"
        self.assertEqual(expected, actual)

    def test_single_context_item_output(self):
        context = {"File": "/home/user/test.vcf"}
        actual = str(VCardError("message", context))
        expected = "message\nFile: /home/user/test.vcf"
        self.assertEqual(expected, actual)

    def test_full_context_output(self):
        context = {
            "File": "/home/user/test.vcf",
            "File line": 120,
            "vCard line": 5,
            "Property": "ADR",
            "Property line": 2,
            "String": "too;few;values;êéè",
        }
        actual = str(VCardError("message", context))
        expected = "\n".join(
            [
                "message",
                "File: /home/user/test.vcf",
                "File line: 120",
                "vCard line: 5",
                "Property: ADR",
                "Property line: 2",
                "String: too;few;values;êéè",
            ]
        )
        self.assertEqual(expected, actual)



from unittest import TestCase

import mock

from vcard import vcard_validator


class TestVcardValidator(TestCase):
    @mock.patch("vcard.vcard_validator.validate_file")
    def test_result_from_validator(self, validate_file_mock):
        validate_file_mock.return_value = "foo"

        validator = vcard_validator.VcardValidator("/some/path", False)

        self.assertEqual(validator.result, "foo")
